@@ -19,4 +19,5 @@ group by
     o_shippriority
 order by
     revenue desc,
-    o_orderdate;
\ No newline at end of file
+    o_orderdate
+limit 10;
