@@ -15,10 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::path::Path;
-use std::path::PathBuf;
+use std::fs;
+use std::io::ErrorKind;
+use std::path::{Path, PathBuf};
 
 use crate::util::{BenchmarkRun, CommonOpt, QueryResult};
+use datafusion::logical_expr::{ExplainFormat, ExplainOption};
 use datafusion::{
     error::{DataFusionError, Result},
     prelude::SessionContext,
@@ -27,7 +29,7 @@ use datafusion_common::exec_datafusion_err;
 use datafusion_common::instant::Instant;
 use structopt::StructOpt;
 
-/// Run the clickbench benchmark
+/// Driver program to run the ClickBench benchmark
 ///
 /// The ClickBench[1] benchmarks are widely cited in the industry and
 /// focus on grouping / aggregation / filtering. This runner uses the
@@ -42,6 +44,14 @@ pub struct RunOpt {
     #[structopt(short, long)]
     query: Option<usize>,
 
+    /// If specified, enables Parquet Filter Pushdown.
+    ///
+    /// Specifically, it enables:
+    /// * `pushdown_filters = true`
+    /// * `reorder_filters = true`
+    #[structopt(long = "pushdown")]
+    pushdown: bool,
+
     /// Common options
     #[structopt(flatten)]
     common: CommonOpt,
@@ -56,12 +66,12 @@ pub struct RunOpt {
     )]
     path: PathBuf,
 
-    /// Path to queries.sql (single file)
+    /// Path to queries directory
     #[structopt(
         parse(from_os_str),
         short = "r",
         long = "queries-path",
-        default_value = "benchmarks/queries/clickbench/queries.sql"
+        default_value = "benchmarks/queries/clickbench/queries"
     )]
     queries_path: PathBuf,
 
@@ -70,49 +80,47 @@ pub struct RunOpt {
     output_path: Option<PathBuf>,
 }
 
-struct AllQueries {
-    queries: Vec<String>,
+/// Get the SQL file path
+pub fn get_query_path(query_dir: &Path, query: usize) -> PathBuf {
+    let mut query_path = query_dir.to_path_buf();
+    query_path.push(format!("q{query}.sql"));
+    query_path
 }
 
-impl AllQueries {
-    fn try_new(path: &Path) -> Result<Self> {
-        // ClickBench has all queries in a single file identified by line number
-        let all_queries = std::fs::read_to_string(path)
-            .map_err(|e| exec_datafusion_err!("Could not open {path:?}: {e}"))?;
-        Ok(Self {
-            queries: all_queries.lines().map(|s| s.to_string()).collect(),
-        })
-    }
-
-    /// Returns the text of query `query_id`
-    fn get_query(&self, query_id: usize) -> Result<&str> {
-        self.queries
-            .get(query_id)
-            .ok_or_else(|| {
-                let min_id = self.min_query_id();
-                let max_id = self.max_query_id();
-                exec_datafusion_err!(
-                    "Invalid query id {query_id}. Must be between {min_id} and {max_id}"
-                )
-            })
-            .map(|s| s.as_str())
-    }
-
-    fn min_query_id(&self) -> usize {
-        0
-    }
-
-    fn max_query_id(&self) -> usize {
-        self.queries.len() - 1
+/// Get the SQL statement from the specified query file
+pub fn get_query_sql(query_path: &Path) -> Result<Option<String>> {
+    if fs::exists(query_path)? {
+        Ok(Some(fs::read_to_string(query_path)?))
+    } else {
+        Ok(None)
     }
 }
+
 impl RunOpt {
     pub async fn run(self) -> Result<()> {
         println!("Running benchmarks with the following options: {self:?}");
-        let queries = AllQueries::try_new(self.queries_path.as_path())?;
+
+        let query_dir_metadata = fs::metadata(&self.queries_path).map_err(|e| {
+            if e.kind() == ErrorKind::NotFound {
+                exec_datafusion_err!(
+                    "Query path '{}' does not exist.",
+                    &self.queries_path.to_str().unwrap()
+                )
+            } else {
+                DataFusionError::External(Box::new(e))
+            }
+        })?;
+
+        if !query_dir_metadata.is_dir() {
+            return Err(exec_datafusion_err!(
+                "Query path '{}' is not a directory.",
+                &self.queries_path.to_str().unwrap()
+            ));
+        }
+
         let query_range = match self.query {
             Some(query_id) => query_id..=query_id,
-            None => queries.min_query_id()..=queries.max_query_id(),
+            None => 0..=usize::MAX,
         };
 
         // configure parquet options
@@ -122,6 +130,12 @@ impl RunOpt {
             // The hits_partitioned dataset specifies string columns
             // as binary due to how it was written. Force it to strings
             parquet_options.binary_as_string = true;
+
+            // Turn on Parquet filter pushdown if requested
+            if self.pushdown {
+                parquet_options.pushdown_filters = true;
+                parquet_options.reorder_filters = true;
+            }
         }
 
         let rt_builder = self.common.runtime_env_builder()?;
@@ -130,8 +144,18 @@ impl RunOpt {
 
         let mut benchmark_run = BenchmarkRun::new();
         for query_id in query_range {
+            let query_path = get_query_path(&self.queries_path, query_id);
+            let Some(sql) = get_query_sql(&query_path)? else {
+                if self.query.is_some() {
+                    return Err(exec_datafusion_err!(
+                        "Could not load query file '{}'.",
+                        &query_path.to_str().unwrap()
+                    ));
+                }
+                break;
+            };
             benchmark_run.start_new_case(&format!("Query {query_id}"));
-            let query_run = self.benchmark_query(&queries, query_id, &ctx).await;
+            let query_run = self.benchmark_query(&sql, query_id, &ctx).await;
             match query_run {
                 Ok(query_results) => {
                     for iter in query_results {
@@ -151,11 +175,10 @@ impl RunOpt {
 
     async fn benchmark_query(
         &self,
-        queries: &AllQueries,
+        sql: &str,
         query_id: usize,
         ctx: &SessionContext,
     ) -> Result<Vec<QueryResult>> {
-        let sql = queries.get_query(query_id)?;
         println!("Q{query_id}: {sql}");
 
         let mut millis = Vec::with_capacity(self.iterations());
@@ -173,7 +196,13 @@ impl RunOpt {
             query_results.push(QueryResult { elapsed, row_count })
         }
         if self.common.debug {
-            ctx.sql(sql).await?.explain(false, false)?.show().await?;
+            ctx.sql(sql)
+                .await?
+                .explain_with_options(
+                    ExplainOption::default().with_format(ExplainFormat::Tree),
+                )?
+                .show()
+                .await?;
         }
         let avg = millis.iter().sum::<f64>() / millis.len() as f64;
         println!("Query {query_id} avg time: {avg:.2} ms");
