@@ -17,6 +17,7 @@
 
 use std::sync::Arc;
 
+use super::record_batch_generator::get_supported_types_columns;
 use crate::fuzz_cases::aggregation_fuzzer::query_builder::QueryBuilder;
 use crate::fuzz_cases::aggregation_fuzzer::{
     AggregationFuzzerBuilder, DatasetGeneratorConfig,
@@ -26,39 +27,35 @@ use arrow::array::{
     types::Int64Type, Array, ArrayRef, AsArray, Int32Array, Int64Array, RecordBatch,
     StringArray,
 };
-use arrow::compute::{concat_batches, SortOptions};
+use arrow::compute::concat_batches;
 use arrow::datatypes::DataType;
 use arrow::util::pretty::pretty_format_batches;
 use arrow_schema::{Field, Schema, SchemaRef};
-use datafusion::common::Result;
 use datafusion::datasource::memory::MemorySourceConfig;
 use datafusion::datasource::source::DataSourceExec;
 use datafusion::datasource::MemTable;
-use datafusion::physical_expr::aggregate::AggregateExprBuilder;
-use datafusion::physical_plan::aggregates::{
-    AggregateExec, AggregateMode, PhysicalGroupBy,
-};
-use datafusion::physical_plan::{collect, displayable, ExecutionPlan};
 use datafusion::prelude::{DataFrame, SessionConfig, SessionContext};
 use datafusion_common::tree_node::{TreeNode, TreeNodeRecursion, TreeNodeVisitor};
-use datafusion_common::HashMap;
+use datafusion_common::{HashMap, Result};
 use datafusion_common_runtime::JoinSet;
-use datafusion_functions_aggregate::sum::sum_udaf;
-use datafusion_physical_expr::expressions::{col, lit, Column};
-use datafusion_physical_expr::PhysicalSortExpr;
-use datafusion_physical_expr_common::sort_expr::LexOrdering;
-use datafusion_physical_plan::InputOrderMode;
-use test_utils::{add_empty_batches, StringBatchGenerator};
-
 use datafusion_execution::memory_pool::FairSpillPool;
 use datafusion_execution::runtime_env::RuntimeEnvBuilder;
 use datafusion_execution::TaskContext;
+use datafusion_functions_aggregate::sum::sum_udaf;
+use datafusion_physical_expr::aggregate::AggregateExprBuilder;
+use datafusion_physical_expr::expressions::{col, lit, Column};
+use datafusion_physical_expr::PhysicalSortExpr;
+use datafusion_physical_plan::aggregates::{
+    AggregateExec, AggregateMode, PhysicalGroupBy,
+};
 use datafusion_physical_plan::metrics::MetricValue;
+use datafusion_physical_plan::InputOrderMode;
+use datafusion_physical_plan::{collect, displayable, ExecutionPlan};
+use test_utils::{add_empty_batches, StringBatchGenerator};
+
 use rand::rngs::StdRng;
 use rand::{random, rng, Rng, SeedableRng};
 
-use super::record_batch_generator::get_supported_types_columns;
-
 // ========================================================================
 //  The new aggregation fuzz tests based on [`AggregationFuzzer`]
 // ========================================================================
@@ -309,13 +306,9 @@ async fn run_aggregate_test(input1: Vec<RecordBatch>, group_by_columns: Vec<&str
     let schema = input1[0].schema();
     let session_config = SessionConfig::new().with_batch_size(50);
     let ctx = SessionContext::new_with_config(session_config);
-    let mut sort_keys = LexOrdering::default();
-    for ordering_col in ["a", "b", "c"] {
-        sort_keys.push(PhysicalSortExpr {
-            expr: col(ordering_col, &schema).unwrap(),
-            options: SortOptions::default(),
-        })
-    }
+    let sort_keys = ["a", "b", "c"].map(|ordering_col| {
+        PhysicalSortExpr::new_default(col(ordering_col, &schema).unwrap())
+    });
 
     let concat_input_record = concat_batches(&schema, &input1).unwrap();
 
@@ -329,7 +322,7 @@ async fn run_aggregate_test(input1: Vec<RecordBatch>, group_by_columns: Vec<&str
     let running_source = DataSourceExec::from_data_source(
         MemorySourceConfig::try_new(&[input1.clone()], schema.clone(), None)
             .unwrap()
-            .try_with_sort_information(vec![sort_keys])
+            .try_with_sort_information(vec![sort_keys.into()])
             .unwrap(),
     );
 
