@@ -123,10 +123,11 @@ impl SimplifyExpressions {
         let name_preserver = NamePreserver::new(&plan);
         let mut rewrite_expr = |expr: Expr| {
             let name = name_preserver.save(&expr);
-            let expr = simplifier.simplify(expr)?;
-            // TODO it would be nice to have a way to know if the expression was simplified
-            // or not. For now conservatively return Transformed::yes
-            Ok(Transformed::yes(name.restore(expr)))
+            let expr = simplifier.simplify_with_cycle_count_transformed(expr)?.0;
+            Ok(Transformed::new_transformed(
+                name.restore(expr.data),
+                expr.transformed,
+            ))
         };
 
         plan.map_expressions(|expr| {
@@ -154,12 +155,12 @@ mod tests {
     use arrow::datatypes::{DataType, Field, Schema};
     use chrono::{DateTime, Utc};
 
-    use crate::optimizer::Optimizer;
     use datafusion_expr::logical_plan::builder::table_scan_with_filters;
     use datafusion_expr::logical_plan::table_scan;
     use datafusion_expr::*;
     use datafusion_functions_aggregate::expr_fn::{max, min};
 
+    use crate::assert_optimized_plan_eq_snapshot;
     use crate::test::{assert_fields_eq, test_table_scan_with_name};
     use crate::OptimizerContext;
 
@@ -179,15 +180,20 @@ mod tests {
             .expect("building plan")
     }
 
-    fn assert_optimized_plan_eq(plan: LogicalPlan, expected: &str) -> Result<()> {
-        // Use Optimizer to do plan traversal
-        fn observe(_plan: &LogicalPlan, _rule: &dyn OptimizerRule) {}
-        let optimizer = Optimizer::with_rules(vec![Arc::new(SimplifyExpressions::new())]);
-        let optimized_plan =
-            optimizer.optimize(plan, &OptimizerContext::new(), observe)?;
-        let formatted_plan = format!("{optimized_plan}");
-        assert_eq!(formatted_plan, expected);
-        Ok(())
+    macro_rules! assert_optimized_plan_equal {
+        (
+            $plan:expr,
+            @ $expected:literal $(,)?
+        ) => {{
+            let rules: Vec<Arc<dyn crate::OptimizerRule + Send + Sync>> = vec![Arc::new(SimplifyExpressions::new())];
+            let optimizer_ctx = OptimizerContext::new();
+            assert_optimized_plan_eq_snapshot!(
+                optimizer_ctx,
+                rules,
+                $plan,
+                @ $expected,
+            )
+        }};
     }
 
     #[test]
@@ -210,9 +216,10 @@ mod tests {
         assert_eq!(1, table_scan.schema().fields().len());
         assert_fields_eq(&table_scan, vec!["a"]);
 
-        let expected = "TableScan: test projection=[a], full_filters=[Boolean(true)]";
-
-        assert_optimized_plan_eq(table_scan, expected)
+        assert_optimized_plan_equal!(
+            table_scan,
+            @ r"TableScan: test projection=[a], full_filters=[Boolean(true)]"
+        )
     }
 
     #[test]
@@ -223,12 +230,13 @@ mod tests {
             .filter(and(col("b").gt(lit(1)), col("b").gt(lit(1))))?
             .build()?;
 
-        assert_optimized_plan_eq(
+        assert_optimized_plan_equal!(
             plan,
-            "\
-	        Filter: test.b > Int32(1)\
-            \n  Projection: test.a\
-            \n    TableScan: test",
+            @ r"
+        Filter: test.b > Int32(1)
+          Projection: test.a
+            TableScan: test
+        "
         )
     }
 
@@ -240,12 +248,13 @@ mod tests {
             .filter(and(col("b").gt(lit(1)), col("b").gt(lit(1))))?
             .build()?;
 
-        assert_optimized_plan_eq(
+        assert_optimized_plan_equal!(
             plan,
-            "\
-	        Filter: test.b > Int32(1)\
-            \n  Projection: test.a\
-            \n    TableScan: test",
+            @ r"
+            Filter: test.b > Int32(1)
+              Projection: test.a
+                TableScan: test
+            "
         )
     }
 
@@ -257,12 +266,13 @@ mod tests {
             .filter(or(col("b").gt(lit(1)), col("b").gt(lit(1))))?
             .build()?;
 
-        assert_optimized_plan_eq(
+        assert_optimized_plan_equal!(
             plan,
-            "\
-            Filter: test.b > Int32(1)\
-            \n  Projection: test.a\
-            \n    TableScan: test",
+            @ r"
+            Filter: test.b > Int32(1)
+              Projection: test.a
+                TableScan: test
+            "
         )
     }
 
@@ -278,12 +288,13 @@ mod tests {
             ))?
             .build()?;
 
-        assert_optimized_plan_eq(
+        assert_optimized_plan_equal!(
             plan,
-            "\
-            Filter: test.a > Int32(5) AND test.b < Int32(6)\
-            \n  Projection: test.a, test.b\
-	        \n    TableScan: test",
+            @ r"
+        Filter: test.a > Int32(5) AND test.b < Int32(6)
+          Projection: test.a, test.b
+            TableScan: test
+        "
         )
     }
 
@@ -296,13 +307,15 @@ mod tests {
             .project(vec![col("a")])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a\
-        \n  Filter: NOT test.c\
-        \n    Filter: test.b\
-        \n      TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a
+          Filter: NOT test.c
+            Filter: test.b
+              TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -315,14 +328,16 @@ mod tests {
             .project(vec![col("a")])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a\
-        \n  Limit: skip=0, fetch=1\
-        \n    Filter: test.c\
-        \n      Filter: NOT test.b\
-        \n        TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a
+          Limit: skip=0, fetch=1
+            Filter: test.c
+              Filter: NOT test.b
+                TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -333,12 +348,14 @@ mod tests {
             .project(vec![col("a")])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a\
-        \n  Filter: NOT test.b AND test.c\
-        \n    TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a
+          Filter: NOT test.b AND test.c
+            TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -349,12 +366,14 @@ mod tests {
             .project(vec![col("a")])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a\
-        \n  Filter: NOT test.b OR NOT test.c\
-        \n    TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a
+          Filter: NOT test.b OR NOT test.c
+            TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -365,12 +384,14 @@ mod tests {
             .project(vec![col("a")])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a\
-        \n  Filter: test.b\
-        \n    TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a
+          Filter: test.b
+            TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -380,11 +401,13 @@ mod tests {
             .project(vec![col("a"), col("d"), col("b").eq(lit(false))])?
             .build()?;
 
-        let expected = "\
-        Projection: test.a, test.d, NOT test.b AS test.b = Boolean(false)\
-        \n  TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Projection: test.a, test.d, NOT test.b AS test.b = Boolean(false)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -398,12 +421,14 @@ mod tests {
             )?
             .build()?;
 
-        let expected = "\
-        Aggregate: groupBy=[[test.a, test.c]], aggr=[[max(test.b) AS max(test.b = Boolean(true)), min(test.b)]]\
-        \n  Projection: test.a, test.c, test.b\
-        \n    TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Aggregate: groupBy=[[test.a, test.c]], aggr=[[max(test.b) AS max(test.b = Boolean(true)), min(test.b)]]
+          Projection: test.a, test.c, test.b
+            TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -421,10 +446,10 @@ mod tests {
         let values = vec![vec![expr1, expr2]];
         let plan = LogicalPlanBuilder::values(values)?.build()?;
 
-        let expected = "\
-        Values: (Int32(3) AS Int32(1) + Int32(2), Int32(1) AS Int32(2) - Int32(1))";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ "Values: (Int32(3) AS Int32(1) + Int32(2), Int32(1) AS Int32(2) - Int32(1))"
+        )
     }
 
     fn get_optimized_plan_formatted(
@@ -481,10 +506,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").gt(lit(10)).not())?
             .build()?;
-        let expected = "Filter: test.d <= Int32(10)\
-            \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d <= Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -494,10 +523,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").gt(lit(10)).and(col("d").lt(lit(100))).not())?
             .build()?;
-        let expected = "Filter: test.d <= Int32(10) OR test.d >= Int32(100)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d <= Int32(10) OR test.d >= Int32(100)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -507,10 +540,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").gt(lit(10)).or(col("d").lt(lit(100))).not())?
             .build()?;
-        let expected = "Filter: test.d <= Int32(10) AND test.d >= Int32(100)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d <= Int32(10) AND test.d >= Int32(100)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -520,10 +557,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").gt(lit(10)).not().not())?
             .build()?;
-        let expected = "Filter: test.d > Int32(10)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d > Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -533,10 +574,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("e").is_null().not())?
             .build()?;
-        let expected = "Filter: test.e IS NOT NULL\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.e IS NOT NULL
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -546,10 +591,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("e").is_not_null().not())?
             .build()?;
-        let expected = "Filter: test.e IS NULL\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.e IS NULL
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -559,11 +608,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").in_list(vec![lit(1), lit(2), lit(3)], false).not())?
             .build()?;
-        let expected =
-            "Filter: test.d != Int32(1) AND test.d != Int32(2) AND test.d != Int32(3)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d != Int32(1) AND test.d != Int32(2) AND test.d != Int32(3)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -573,11 +625,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").in_list(vec![lit(1), lit(2), lit(3)], true).not())?
             .build()?;
-        let expected =
-            "Filter: test.d = Int32(1) OR test.d = Int32(2) OR test.d = Int32(3)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d = Int32(1) OR test.d = Int32(2) OR test.d = Int32(3)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -588,10 +643,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(qual.not())?
             .build()?;
-        let expected = "Filter: test.d < Int32(1) OR test.d > Int32(10)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d < Int32(1) OR test.d > Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -602,10 +661,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(qual.not())?
             .build()?;
-        let expected = "Filter: test.d >= Int32(1) AND test.d <= Int32(10)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d >= Int32(1) AND test.d <= Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -622,10 +685,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("a").like(col("b")).not())?
             .build()?;
-        let expected = "Filter: test.a NOT LIKE test.b\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.a NOT LIKE test.b
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -642,10 +709,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("a").not_like(col("b")).not())?
             .build()?;
-        let expected = "Filter: test.a LIKE test.b\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.a LIKE test.b
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -662,10 +733,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("a").ilike(col("b")).not())?
             .build()?;
-        let expected = "Filter: test.a NOT ILIKE test.b\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.a NOT ILIKE test.b
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -675,10 +750,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(binary_expr(col("d"), Operator::IsDistinctFrom, lit(10)).not())?
             .build()?;
-        let expected = "Filter: test.d IS NOT DISTINCT FROM Int32(10)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d IS NOT DISTINCT FROM Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -688,10 +767,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(binary_expr(col("d"), Operator::IsNotDistinctFrom, lit(10)).not())?
             .build()?;
-        let expected = "Filter: test.d IS DISTINCT FROM Int32(10)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.d IS DISTINCT FROM Int32(10)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -713,11 +796,14 @@ mod tests {
 
         // before simplify: t1.a + CAST(Int64(1), UInt32) = t2.a + CAST(Int64(2), UInt32)
         // after simplify: t1.a + UInt32(1) = t2.a + UInt32(2) AS t1.a + Int64(1) = t2.a + Int64(2)
-        let expected = "Inner Join: t1.a + UInt32(1) = t2.a + UInt32(2)\
-            \n  TableScan: t1\
-            \n  TableScan: t2";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Inner Join: t1.a + UInt32(1) = t2.a + UInt32(2)
+          TableScan: t1
+          TableScan: t2
+        "
+        )
     }
 
     #[test]
@@ -727,10 +813,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").is_not_null())?
             .build()?;
-        let expected = "Filter: Boolean(true)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: Boolean(true)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -740,10 +830,14 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan)
             .filter(col("d").is_null())?
             .build()?;
-        let expected = "Filter: Boolean(false)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: Boolean(false)
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -760,10 +854,13 @@ mod tests {
             )?
             .build()?;
 
-        let expected = "Aggregate: groupBy=[[GROUPING SETS ((Int32(43) AS age, test.a), (Boolean(false) AS cond), (test.d AS e, Int32(3) AS Int32(1) + Int32(2)))]], aggr=[[]]\
-        \n  TableScan: test";
-
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Aggregate: groupBy=[[GROUPING SETS ((Int32(43) AS age, test.a), (Boolean(false) AS cond), (test.d AS e, Int32(3) AS Int32(1) + Int32(2)))]], aggr=[[]]
+          TableScan: test
+        "
+        )
     }
 
     #[test]
@@ -778,19 +875,27 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .filter(binary_expr(col("a"), Operator::RegexMatch, lit(".*")))?
             .build()?;
-        let expected = "Filter: test.a IS NOT NULL\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)?;
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: test.a IS NOT NULL
+          TableScan: test
+        "
+        )?;
 
         // Test `!= ".*"` transforms to checking if the column is empty
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .filter(binary_expr(col("a"), Operator::RegexNotMatch, lit(".*")))?
             .build()?;
-        let expected = "Filter: test.a = Utf8(\"\")\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)?;
+        assert_optimized_plan_equal!(
+            plan,
+            @ r#"
+        Filter: test.a = Utf8("")
+          TableScan: test
+        "#
+        )?;
 
         // Test case-insensitive versions
 
@@ -798,18 +903,26 @@ mod tests {
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .filter(binary_expr(col("b"), Operator::RegexIMatch, lit(".*")))?
             .build()?;
-        let expected = "Filter: Boolean(true)\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)?;
+        assert_optimized_plan_equal!(
+            plan,
+            @ r"
+        Filter: Boolean(true)
+          TableScan: test
+        "
+        )?;
 
         // Test `!~ ".*"` (case-insensitive) transforms to checking if the column is empty
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .filter(binary_expr(col("a"), Operator::RegexNotIMatch, lit(".*")))?
             .build()?;
-        let expected = "Filter: test.a = Utf8(\"\")\
-        \n  TableScan: test";
 
-        assert_optimized_plan_eq(plan, expected)
+        assert_optimized_plan_equal!(
+            plan,
+            @ r#"
+        Filter: test.a = Utf8("")
+          TableScan: test
+        "#
+        )
     }
 }
