@@ -38,13 +38,13 @@ use crate::expressions::Literal;
 use crate::PhysicalExpr;
 
 use arrow::array::{Array, RecordBatch};
-use arrow::datatypes::{DataType, Schema};
+use arrow::datatypes::{DataType, FieldRef, Schema};
 use datafusion_common::{internal_err, Result, ScalarValue};
 use datafusion_expr::interval_arithmetic::Interval;
 use datafusion_expr::sort_properties::ExprProperties;
 use datafusion_expr::type_coercion::functions::data_types_with_scalar_udf;
 use datafusion_expr::{
-    expr_vec_fmt, ColumnarValue, ReturnTypeArgs, ScalarFunctionArgs, ScalarUDF,
+    expr_vec_fmt, ColumnarValue, ReturnFieldArgs, ScalarFunctionArgs, ScalarUDF,
 };
 
 /// Physical expression of a scalar function
@@ -53,8 +53,7 @@ pub struct ScalarFunctionExpr {
     fun: Arc<ScalarUDF>,
     name: String,
     args: Vec<Arc<dyn PhysicalExpr>>,
-    return_type: DataType,
-    nullable: bool,
+    return_field: FieldRef,
 }
 
 impl Debug for ScalarFunctionExpr {
@@ -63,7 +62,7 @@ impl Debug for ScalarFunctionExpr {
             .field("fun", &"<FUNC>")
             .field("name", &self.name)
             .field("args", &self.args)
-            .field("return_type", &self.return_type)
+            .field("return_field", &self.return_field)
             .finish()
     }
 }
@@ -74,14 +73,13 @@ impl ScalarFunctionExpr {
         name: &str,
         fun: Arc<ScalarUDF>,
         args: Vec<Arc<dyn PhysicalExpr>>,
-        return_type: DataType,
+        return_field: FieldRef,
     ) -> Self {
         Self {
             fun,
             name: name.to_owned(),
             args,
-            return_type,
-            nullable: true,
+            return_field,
         }
     }
 
@@ -92,18 +90,17 @@ impl ScalarFunctionExpr {
         schema: &Schema,
     ) -> Result<Self> {
         let name = fun.name().to_string();
-        let arg_types = args
+        let arg_fields = args
             .iter()
-            .map(|e| e.data_type(schema))
+            .map(|e| e.return_field(schema))
             .collect::<Result<Vec<_>>>()?;
 
         // verify that input data types is consistent with function's `TypeSignature`
-        data_types_with_scalar_udf(&arg_types, &fun)?;
-
-        let nullables = args
+        let arg_types = arg_fields
             .iter()
-            .map(|e| e.nullable(schema))
-            .collect::<Result<Vec<_>>>()?;
+            .map(|f| f.data_type().clone())
+            .collect::<Vec<_>>();
+        data_types_with_scalar_udf(&arg_types, &fun)?;
 
         let arguments = args
             .iter()
@@ -113,18 +110,16 @@ impl ScalarFunctionExpr {
                     .map(|literal| literal.value())
             })
             .collect::<Vec<_>>();
-        let ret_args = ReturnTypeArgs {
-            arg_types: &arg_types,
+        let ret_args = ReturnFieldArgs {
+            arg_fields: &arg_fields,
             scalar_arguments: &arguments,
-            nullables: &nullables,
         };
-        let (return_type, nullable) = fun.return_type_from_args(ret_args)?.into_parts();
+        let return_field = fun.return_field_from_args(ret_args)?;
         Ok(Self {
             fun,
             name,
             args,
-            return_type,
-            nullable,
+            return_field,
         })
     }
 
@@ -145,16 +140,21 @@ impl ScalarFunctionExpr {
 
     /// Data type produced by this expression
     pub fn return_type(&self) -> &DataType {
-        &self.return_type
+        self.return_field.data_type()
     }
 
     pub fn with_nullable(mut self, nullable: bool) -> Self {
-        self.nullable = nullable;
+        self.return_field = self
+            .return_field
+            .as_ref()
+            .clone()
+            .with_nullable(nullable)
+            .into();
         self
     }
 
     pub fn nullable(&self) -> bool {
-        self.nullable
+        self.return_field.is_nullable()
     }
 }
 
@@ -171,11 +171,11 @@ impl PhysicalExpr for ScalarFunctionExpr {
     }
 
     fn data_type(&self, _input_schema: &Schema) -> Result<DataType> {
-        Ok(self.return_type.clone())
+        Ok(self.return_field.data_type().clone())
     }
 
     fn nullable(&self, _input_schema: &Schema) -> Result<bool> {
-        Ok(self.nullable)
+        Ok(self.return_field.is_nullable())
     }
 
     fn evaluate(&self, batch: &RecordBatch) -> Result<ColumnarValue> {
@@ -185,6 +185,12 @@ impl PhysicalExpr for ScalarFunctionExpr {
             .map(|e| e.evaluate(batch))
             .collect::<Result<Vec<_>>>()?;
 
+        let arg_fields = self
+            .args
+            .iter()
+            .map(|e| e.return_field(batch.schema_ref()))
+            .collect::<Result<Vec<_>>>()?;
+
         let input_empty = args.is_empty();
         let input_all_scalar = args
             .iter()
@@ -193,8 +199,9 @@ impl PhysicalExpr for ScalarFunctionExpr {
         // evaluate the function
         let output = self.fun.invoke_with_args(ScalarFunctionArgs {
             args,
+            arg_fields,
             number_rows: batch.num_rows(),
-            return_type: &self.return_type,
+            return_field: Arc::clone(&self.return_field),
         })?;
 
         if let ColumnarValue::Array(array) = &output {
@@ -214,6 +221,10 @@ impl PhysicalExpr for ScalarFunctionExpr {
         Ok(output)
     }
 
+    fn return_field(&self, _input_schema: &Schema) -> Result<FieldRef> {
+        Ok(Arc::clone(&self.return_field))
+    }
+
     fn children(&self) -> Vec<&Arc<dyn PhysicalExpr>> {
         self.args.iter().collect()
     }
@@ -222,15 +233,12 @@ impl PhysicalExpr for ScalarFunctionExpr {
         self: Arc<Self>,
         children: Vec<Arc<dyn PhysicalExpr>>,
     ) -> Result<Arc<dyn PhysicalExpr>> {
-        Ok(Arc::new(
-            ScalarFunctionExpr::new(
-                &self.name,
-                Arc::clone(&self.fun),
-                children,
-                self.return_type().clone(),
-            )
-            .with_nullable(self.nullable),
-        ))
+        Ok(Arc::new(ScalarFunctionExpr::new(
+            &self.name,
+            Arc::clone(&self.fun),
+            children,
+            Arc::clone(&self.return_field),
+        )))
     }
 
     fn evaluate_bounds(&self, children: &[&Interval]) -> Result<Interval> {
