@@ -16,7 +16,7 @@
 // under the License.
 
 //! Special channel construction to distribute data from various inputs into N outputs
-//! minimizing buffering but preventing deadlocks when repartitoning
+//! minimizing buffering but preventing deadlocks when repartitioning
 //!
 //! # Design
 //!
@@ -167,11 +167,11 @@ impl<T> Drop for DistributionSender<T> {
             //
             // If the last sender is dropped first, `state.data` will still exists and the sender side decrements the
             // signal. The receiver side then MUST check the `n_senders` counter during the section and if it is zero,
-            // it inferres that it is dropped afterwards and MUST NOT decrement the counter.
+            // it infers that it is dropped afterwards and MUST NOT decrement the counter.
             //
-            // If the receiver end is dropped first, it will inferr -- based on `n_senders` -- that there are still
+            // If the receiver end is dropped first, it will infer -- based on `n_senders` -- that there are still
             // senders and it will decrement the `empty_channels` counter. It will also set `data` to `None`. The sender
-            // side will then see that `data` is `None` and can therefore inferr that the receiver end was dropped, and
+            // side will then see that `data` is `None` and can therefore infer that the receiver end was dropped, and
             // hence it MUST NOT decrement the `empty_channels` counter.
             if state
                 .data
@@ -203,7 +203,7 @@ pub struct SendFuture<'a, T> {
     element: Box<Option<T>>,
 }
 
-impl<'a, T> Future for SendFuture<'a, T> {
+impl<T> Future for SendFuture<'_, T> {
     type Output = Result<(), SendError<T>>;
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
@@ -295,7 +295,7 @@ pub struct RecvFuture<'a, T> {
     rdy: bool,
 }
 
-impl<'a, T> Future for RecvFuture<'a, T> {
+impl<T> Future for RecvFuture<'_, T> {
     type Output = Option<T>;
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
