@@ -18,12 +18,14 @@
 /// Implementation of sqllogictest for datafusion.
 mod conversion;
 mod datafusion_engine;
+mod datafusion_substrait_roundtrip_engine;
 mod output;
 
 pub use datafusion_engine::convert_batches;
 pub use datafusion_engine::convert_schema_to_types;
 pub use datafusion_engine::DFSqlLogicTestError;
 pub use datafusion_engine::DataFusion;
+pub use datafusion_substrait_roundtrip_engine::DataFusionSubstraitRoundTrip;
 pub use output::DFColumnType;
 pub use output::DFOutput;
 
